@@ -167,7 +167,7 @@ fn inherit_own_workspace_fields() {
 [FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
 [UPLOADING] foo v1.2.3 ([ROOT]/foo)
 [UPLOADED] foo v1.2.3 to registry `crates-io`
-[NOTE] waiting for `foo v1.2.3` to be available at registry `crates-io`.
+[NOTE] waiting for foo v1.2.3 to be available at registry `crates-io`.
 You may press ctrl-c to skip waiting; the crate should be available shortly.
 [PUBLISHED] foo v1.2.3 at registry `crates-io`
 
@@ -206,10 +206,20 @@ You may press ctrl-c to skip waiting; the crate should be available shortly.
             ".cargo_vcs_info.json",
             "bar.txt",
         ],
-        &[(
+        [(
             "Cargo.toml",
-            &format!(
-                r#"{}
+            str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
 [package]
 edition = "2018"
 rust-version = "1.60"
@@ -241,9 +251,8 @@ repository = "https://github.com/example/example"
 [[bin]]
 name = "foo"
 path = "src/main.rs"
-"#,
-                cargo::core::manifest::MANIFEST_PREAMBLE
-            ),
+
+"##]],
         )],
     );
 }
@@ -325,7 +334,7 @@ See https://doc.rust-lang.org/cargo/reference/manifest.html#package-metadata for
 [FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
 [UPLOADING] bar v0.2.0 ([ROOT]/foo)
 [UPLOADED] bar v0.2.0 to registry `crates-io`
-[NOTE] waiting for `bar v0.2.0` to be available at registry `crates-io`.
+[NOTE] waiting for bar v0.2.0 to be available at registry `crates-io`.
 You may press ctrl-c to skip waiting; the crate should be available shortly.
 [PUBLISHED] bar v0.2.0 at registry `crates-io`
 
@@ -385,10 +394,20 @@ You may press ctrl-c to skip waiting; the crate should be available shortly.
         "#,
         "bar-0.2.0.crate",
         &["Cargo.toml", "Cargo.toml.orig", "Cargo.lock", "src/main.rs"],
-        &[(
+        [(
             "Cargo.toml",
-            &format!(
-                r#"{}
+            str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
 [package]
 edition = "2015"
 name = "bar"
@@ -414,9 +433,8 @@ version = "0.5.2"
 
 [build-dependencies.dep-build]
 version = "0.8"
-"#,
-                cargo::core::manifest::MANIFEST_PREAMBLE
-            ),
+
+"##]],
         )],
     );
 }
@@ -448,7 +466,7 @@ fn inherit_own_detailed_dependencies() {
         .build();
 
     Package::new("dep", "0.1.2")
-        .feature("testing", &vec![])
+        .feature("testing", &[])
         .publish();
 
     p.cargo("check")
@@ -483,7 +501,7 @@ See https://doc.rust-lang.org/cargo/reference/manifest.html#package-metadata for
 [FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
 [UPLOADING] bar v0.2.0 ([ROOT]/foo)
 [UPLOADED] bar v0.2.0 to registry `crates-io`
-[NOTE] waiting for `bar v0.2.0` to be available at registry `crates-io`.
+[NOTE] waiting for bar v0.2.0 to be available at registry `crates-io`.
 You may press ctrl-c to skip waiting; the crate should be available shortly.
 [PUBLISHED] bar v0.2.0 at registry `crates-io`
 
@@ -525,10 +543,20 @@ You may press ctrl-c to skip waiting; the crate should be available shortly.
         "#,
         "bar-0.2.0.crate",
         &["Cargo.toml", "Cargo.toml.orig", "Cargo.lock", "src/main.rs"],
-        &[(
+        [(
             "Cargo.toml",
-            &format!(
-                r#"{}
+            str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
 [package]
 edition = "2015"
 name = "bar"
@@ -549,9 +577,8 @@ path = "src/main.rs"
 [dependencies.dep]
 version = "0.1.2"
 features = ["testing"]
-"#,
-                cargo::core::manifest::MANIFEST_PREAMBLE
-            ),
+
+"##]],
         )],
     );
 }
@@ -735,7 +762,7 @@ See https://doc.rust-lang.org/cargo/reference/manifest.html#the-license-and-lice
 [FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
 [UPLOADING] bar v1.2.3 ([ROOT]/foo/bar)
 [UPLOADED] bar v1.2.3 to registry `crates-io`
-[NOTE] waiting for `bar v1.2.3` to be available at registry `crates-io`.
+[NOTE] waiting for bar v1.2.3 to be available at registry `crates-io`.
 You may press ctrl-c to skip waiting; the crate should be available shortly.
 [PUBLISHED] bar v1.2.3 at registry `crates-io`
 
@@ -776,10 +803,20 @@ You may press ctrl-c to skip waiting; the crate should be available shortly.
             ".cargo_vcs_info.json",
             "bar.txt",
         ],
-        &[(
+        [(
             "Cargo.toml",
-            &format!(
-                r#"{}
+            str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
 [package]
 edition = "2018"
 rust-version = "1.60"
@@ -814,9 +851,8 @@ repository = "https://github.com/example/example"
 [[bin]]
 name = "bar"
 path = "src/main.rs"
-"#,
-                cargo::core::manifest::MANIFEST_PREAMBLE
-            ),
+
+"##]],
         )],
     );
 }
@@ -900,7 +936,7 @@ See https://doc.rust-lang.org/cargo/reference/manifest.html#package-metadata for
 [FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
 [UPLOADING] bar v0.2.0 ([ROOT]/foo/bar)
 [UPLOADED] bar v0.2.0 to registry `crates-io`
-[NOTE] waiting for `bar v0.2.0` to be available at registry `crates-io`.
+[NOTE] waiting for bar v0.2.0 to be available at registry `crates-io`.
 You may press ctrl-c to skip waiting; the crate should be available shortly.
 [PUBLISHED] bar v0.2.0 at registry `crates-io`
 
@@ -960,10 +996,20 @@ You may press ctrl-c to skip waiting; the crate should be available shortly.
         "#,
         "bar-0.2.0.crate",
         &["Cargo.toml", "Cargo.toml.orig", "Cargo.lock", "src/main.rs"],
-        &[(
+        [(
             "Cargo.toml",
-            &format!(
-                r#"{}
+            str![[r##"
+# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
+#
+# When uploading crates to the registry Cargo will automatically
+# "normalize" Cargo.toml files for maximal compatibility
+# with all versions of Cargo and also rewrite `path` dependencies
+# to registry (e.g., crates.io) dependencies.
+#
+# If you are reading this file be aware that the original Cargo.toml
+# will likely look very different (and much more reasonable).
+# See Cargo.toml.orig for the original contents.
+
 [package]
 edition = "2015"
 name = "bar"
@@ -989,9 +1035,8 @@ version = "0.5.2"
 
 [build-dependencies.dep-build]
 version = "0.8"
-"#,
-                cargo::core::manifest::MANIFEST_PREAMBLE
-            ),
+
+"##]],
         )],
     );
 }
@@ -1504,7 +1549,7 @@ fn warn_inherit_def_feat_true_member_def_feat_false() {
 "#]]).run();
 }
 
-#[cargo_test(nightly, reason = "edition2024 is not stable")]
+#[cargo_test]
 fn warn_inherit_def_feat_true_member_def_feat_false_2024_edition() {
     Package::new("dep", "0.1.0")
         .feature("default", &["fancy_dep"])
@@ -1518,8 +1563,6 @@ fn warn_inherit_def_feat_true_member_def_feat_false_2024_edition() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["edition2024"]
-
             [package]
             name = "bar"
             version = "0.2.0"
@@ -1538,7 +1581,6 @@ fn warn_inherit_def_feat_true_member_def_feat_false_2024_edition() {
         .build();
 
     p.cargo("check")
-        .masquerade_as_nightly_cargo(&["edition2024"])
         .with_status(101)
         .with_stderr_data(str![[r#"
 [ERROR] failed to parse manifest at `[ROOT]/foo/Cargo.toml`
@@ -1599,7 +1641,7 @@ fn warn_inherit_simple_member_def_feat_false() {
 "#]]).run();
 }
 
-#[cargo_test(nightly, reason = "edition2024 is not stable")]
+#[cargo_test]
 fn warn_inherit_simple_member_def_feat_false_2024_edition() {
     Package::new("dep", "0.1.0")
         .feature("default", &["fancy_dep"])
@@ -1613,8 +1655,6 @@ fn warn_inherit_simple_member_def_feat_false_2024_edition() {
         .file(
             "Cargo.toml",
             r#"
-            cargo-features = ["edition2024"]
-
             [package]
             name = "bar"
             version = "0.2.0"
@@ -1633,7 +1673,6 @@ fn warn_inherit_simple_member_def_feat_false_2024_edition() {
         .build();
 
     p.cargo("check")
-        .masquerade_as_nightly_cargo(&["edition2024"])
         .with_status(101)
         .with_stderr_data(str![[r#"
 [ERROR] failed to parse manifest at `[ROOT]/foo/Cargo.toml`
